#!/usr/bin/env python

# Author: Thomas Liu <tliu@redhat.com>
# Author: Dan Walsh <dwalsh@redhat.com>

import _policy
import selinux

TYPE = _policy.TYPE
ROLE = _policy.ROLE
ATTRIBUTE = _policy.ATTRIBUTE
PORT = _policy.PORT
USER = _policy.USER
BOOLEAN = _policy.BOOLEAN

ALLOW = 'allow'
AUDITALLOW = 'auditallow'
NEVERALLOW = 'neverallow'
DONTAUDIT = 'dontaudit'
SOURCE = 'source'
TARGET = 'target'
PERMS = 'permlist'
CLASS = 'class'
TRANSITION = 'transition'
ROLE_ALLOW = 'role_allow'

def policy(policy_file):
    _policy.policy(policy_file)

policy_file = selinux.selinux_current_policy_path()
policy(policy_file)

def search(types, info = {} ):
    valid_types = [ALLOW, AUDITALLOW, NEVERALLOW, DONTAUDIT, TRANSITION, ROLE_ALLOW]
    for type in types:
        if type not in valid_types:
            raise ValueError("Type has to be in %s" % valid_types)
        info[type] = True

    perms = []
    if PERMS in info:
        perms = info[PERMS]
        info[PERMS] = ",".join(info[PERMS])

    dict_list = _policy.search(info)
    if dict_list and len(perms) != 0:
        dict_list = filter(lambda x: _dict_has_perms(x, perms), dict_list)
    return dict_list

def _dict_has_perms(dict, perms):
    for perm in perms:
        if perm not in dict[PERMS]:
            return False
    return True

def info(setype, name=None):
    dict_list = _policy.info(setype, name)
    return dict_list

booleans_dict = None
def gen_bool_dict(path="/usr/share/selinux/devel/policy.xml"):
        global booleans_dict
        if booleans_dict:
            return booleans_dict
	import xml.etree.ElementTree
	import re
	booleans_dict = {}
	try:
		tree = xml.etree.ElementTree.parse(path)
		for l in  tree.findall("layer"):
			for m in  l.findall("module"):
				for b in  m.findall("tunable"):
					desc = b.find("desc").find("p").text.strip("\n")
					desc = re.sub("\n", " ", desc)
					booleans_dict[b.get('name')] = (m.get("name"), b.get('dftval'), desc)
				for b in  m.findall("bool"):
					desc = b.find("desc").find("p").text.strip("\n")
					desc = re.sub("\n", " ", desc)
					booleans_dict[b.get('name')] = (m.get("name"), b.get('dftval'), desc)
			for i in  tree.findall("bool"):
				desc = i.find("desc").find("p").text.strip("\n")
				desc = re.sub("\n", " ", desc)
				booleans_dict[i.get('name')] = ("global", i.get('dftval'), desc)
		for i in  tree.findall("tunable"):
			desc = i.find("desc").find("p").text.strip("\n")
			desc = re.sub("\n", " ", desc)
			booleans_dict[i.get('name')] = ("global", i.get('dftval'), desc)
	except IOError, e:
		pass
	return booleans_dict

def boolean_category(boolean):
    booleans_dict = gen_bool_dict()
    if boolean in booleans_dict:
        return _(booleans_dict[boolean][0])
    else:
        return _("unknown")

def boolean_desc(boolean):
       booleans_dict = gen_bool_dict()
       if boolean in booleans_dict:
              return _(booleans_dict[boolean][2])
       else:
              return boolean

def get_os_version():
    os_version = ""
    pkg_name = "selinux-policy"
    try:
	import commands
	rc, output = commands.getstatusoutput("rpm -q '%s'" % pkg_name)
	if rc == 0:
	    os_version = output.split(".")[-2]
    except:
	os_version = ""

    if os_version[0:2] == "fc":
	os_version = "Fedora"+os_version[2:]
    elif os_version[0:2] == "el":
	os_version = "RHEL"+os_version[2:]
    else:
	os_version = ""

    return os_version
